@@ -27,13 +27,14 @@
 #define GLOBAL_HDR_VERSION_0	0
 #define GLOBAL_HDR_VERSION_1	1
 #define GLOBAL_HDR_VERSION_2	2
-	/* version 2 adds encoding of holes and a change to on-tape inventory format.
+#define GLOBAL_HDR_VERSION_3	3
+	/* version 3 uses the full 32-bit inode generation number in direnthdr_t.
+	 * version 2 adds encoding of holes and a change to on-tape inventory format.
 	 * version 1 adds extended file attribute dumping.
 	 * version 0 xfsrestore can't handle media produced
 	 * by version 1 xfsdump. 
 	 */
-#define GLOBAL_HDR_VERSION	GLOBAL_HDR_VERSION_2
-#define GLOBAL_HDR_VERSION_PREV	1
+#define GLOBAL_HDR_VERSION	GLOBAL_HDR_VERSION_3
 
 #define GLOBAL_HDR_STRING_SZ	0x100
 #define GLOBAL_HDR_TIME_SZ	4
@@ -42,15 +43,15 @@
 struct global_hdr {
 	char gh_magic[ GLOBAL_HDR_MAGIC_SZ ];		/*   8    8 */
 		/* unique signature of xfsdump */
-	u_int32_t gh_version;				/*   4    c */
+	uint32_t gh_version;				/*   4    c */
 		/* header version */
-	u_int32_t gh_checksum;				/*   4   10 */
+	uint32_t gh_checksum;				/*   4   10 */
 		/* 32-bit unsigned additive inverse of entire header */
 	time32_t gh_timestamp;				/*   4   14 */
 		/* time32_t of dump */
 	char gh_pad1[ 4 ];				/*   4   18 */
 		/* alignment */
-	u_int64_t gh_ipaddr;				/*   8   20 */
+	uint64_t gh_ipaddr;				/*   8   20 */
 		/* from gethostid(2), room for expansion */
 	uuid_t gh_dumpid;				/*  10   30 */
 		/* ID of dump session	 */
@@ -72,7 +73,7 @@ typedef struct global_hdr global_hdr_t;
 /* used by main( ) to allocate and populate a global header template.
  * drive managers will copy this into the write header.
  */
-extern global_hdr_t * global_hdr_alloc( intgen_t argc, char *argv[ ] );
+extern global_hdr_t * global_hdr_alloc( int argc, char *argv[ ] );
 
 
 /* used by main( ) to free the global header template after drive ini.
@@ -96,7 +97,7 @@ extern bool_t global_hdr_checksum_check( global_hdr_t *hdrp );
  * else return BOOL_FALSE
  */
 
-extern bool_t global_version_check( u_int32_t version );
+extern bool_t global_version_check( uint32_t version );
 
 
 #endif /* GLOBAL_H */
